import re
from datetime import datetime
from .driver import Driver
from .settings import Settings
from .user import User
from PyInquirer import prompt
from PyInquirer import Validator, ValidationError
##
from .validators import AmountValidator, MonthValidator, LimitValidator, PriceValidator, NumberValidator, TimeValidator, DateValidator, DurationValidator, PromoDurationValidator, ExpirationValidator, ListValidator
from . import remote as Remote
from .file import File, Folder, Google_File, Google_Folder

class Discount:
    """OnlyFans discount class"""

    def __init__(self):
        """OnlyFans discount object"""

        # amount in percent
        self.amount = None
        # number of months (1-12)
        self.months = None
        # the recipient username
        self.username = None
        # prevents double prompts
        self.gotten = False

    def apply(self):
        """
        Applies the discounted amount to the recipient username via Driver.discount_user

        If the targeted username is one of the matching keywords then all of the 
        matching recipients will be discounted. Values are determined by runtime args or prompted
        for.

        """

        # ensure the discount has non default values
        self.get()
        if not self.gotten:
            Settings.err_print("Unable to apply discount")
            return
        # skip prompt if disabled
        if Settings.is_prompt():
            if not Settings.prompt("Discount"): return
        Settings.maybe_print("discounting: {}".format(self.username))
        # create new or find default browser
        driver = Driver.get_driver()
        if self.username.lower() == "all":
            users = User.get_all_users(driver=driver)
        elif self.username.lower() == "recent":
            users = User.get_recent_users(driver=driver)
        elif self.username.lower() == "favorite":
            users = User.get_favorite_users(driver=driver)
        elif self.username.lower() == "new":
            users = User.get_new_users(driver=driver)
        else: users = [self]
        for user in users:
            self.username = user.username
            driver.discount_user(discount=self)

    @staticmethod
    def create():
        """Create and apply a discount from args or prompts"""

        discount = Discount()
        discount.apply()

    def get(self):
        """Update the discount object's default values"""

        if self.gotten: return
        gotten = self.get_username()
        gotten = self.get_amount()
        gotten = self.get_months()
        self.gotten = True

    def get_amount(self):
        """
        Populate and get the amount value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        int
            the discounted amount to apply

        """

        if self.amount: return self.amount
        # retrieve from args and return if exists
        amount = Settings.get_amount() or None
        if amount: 
            self.amount = amount
            return amount
        # prompt skip
        if not Settings.prompt("amount"): return None
        question = {
            'type': 'input',
            'name': 'amount',
            'message': 'Amount:',
            'validate': AmountValidator,
            'filter': lambda val: int(myround(int(val)))
        }
        amount = prompt(question)["amount"]
        if not Settings.confirm(amount): return self.get_amount()
        self.amount = amount
        return self.amount

    def get_months(self):
        """
        Populate and get the months value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        int
            the number of months to discount for

        """

        if self.months: return self.months
        # retrieve from args and return if exists
        months = Settings.get_months() or None
        if months: 
            self.months = months
            return months
        # prompt skip
        if not Settings.prompt("months"): return None
        question = {
            'type': 'input',
            'name': 'months',
            'message': 'Months:',
            'validate': MonthValidator,
            'filter': lambda val: int(val)
        }
        months = prompt(question)["months"]
        if not Settings.confirm(months): return self.get_months()
        self.months = months
        return self.months

    def get_username(self):
        """
        Populate and get the username value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        str
            the username to discount

        """

        if self.username: return self.username
        self.username = User.select_user()
        return self.username

    def grandfatherer(self, users=[]):
        """
        Executes the 'Grandfather' discount model

        If users is empty it is populated with users from the 'Grandfather' OnlyFans list in 
        the account. All 'Grandfather'ed users are provided with the max discount for the max months.

        Parameters
        ----------
        users : list
            list of users to 'Grandfather'

        """

        from .driver import Driver
        if len(users) == 0:
            users = User.get_users_by_list(name="grandfathered", driver=Driver.get_driver())
        print("Discount - Grandfathering: {} users".format(len(users)))
        from .validators import DISCOUNT_MAX_MONTHS, DISCOUNT_MAX_AMOUNT
        self.months = DISCOUNT_MAX_MONTHS
        self.amount = DISCOUNT_MAX_AMOUNT
        # apply discount to all users
        for user in users:
            self.username = user.username
            print("Grandfathering: {}".format(self.username))
            try:
                Driver.get_driver().discount_user(discount=self)
            except Exception as e:
                print(e)

########################################################################################

class Message():
    """OnlyFans message (and post) class"""

    def __init__(self):
        """
        OnlyFans message and post object

        A post is just a message on a profile with different options made available.

        """

        # universal message variables
        self.text = None
        self.files = []
        self.keywords = []
        self.performers = []
        self.hasPerformers = False # used to flag files from performer folders
        ## message only variables
        self.price = None # $3 - $100
        self.users = [] # users to send to
        ## post only variables
        self.expiration = None
        self.poll = None
        self.schedule = None
        ##
        self.gotten = False

    def backup_files(self):
        """Backs up files"""

        for file in self.get_files():
            file.backup()

    def delete_files(self):
        """Deletes files"""

        for file in self.get_files():
            file.delete()

    def cleanup_files(self):
        """Processes files after a successful message or post by backing them up then deleting them"""

        self.backup_files()
        self.delete_files()

    @staticmethod
    def format_keywords(keywords):
        """
        Formats the list provided into a combined string with a # in front of each value

        Parameters
        ----------
        keywords : list
            List of keywords as strings

        Returns
        -------
        str
            The generated keywords into a string
        """

        if len(keywords) > 0: return " #{}".format(" #".join(keywords))
        return ""

    @staticmethod
    def format_performers(performers): # spaced added after @ to close performer search modal
        """
        Formats the list provided into a combined string with an @ in front of each value

        Parameters
        ----------
        performers : list
            List of performers usernames as strings

        Returns
        -------
        str
            The generated performers into a string
        """

        if len(performers) > 0: return " w/ @{} ".format(" @".join(performers))
        return ""
            

    def format_text(self):
        """Formats self.text with the provided keywords and performers"""

        return "{}{}{}".format(self.get_text(),
            Message.format_performers(self.get_performers()),
            Message.format_keywords(self.get_keywords())).strip()

    def get_keywords(self):
        """
        Gets the keywords value if not none else sets it from args or prompts.

        Returns
        -------
        list
            The keyword strings in a list

        """

        if str(self.keywords) == "unset": return []
        # if self.keywords: return self.keywords
        if len(self.keywords) > 0: return self.keywords
        # retrieve from args and return if exists
        keywords = Settings.get_keywords() or []
        if len(keywords) > 0: return keywords
        if not Settings.prompt("keywords"):
            self.keywords = "unset" # used to skip prompting for value in future
            return []
        question = {
            'type': 'input',
            'name': 'keywords',
            'message': 'Keywords:',
            'validate': ListValidator
        }
        keywords = prompt(question)["keywords"]
        keywords = [n.strip() for n in keywords.split(",")]
        # confirm keywords
        if not Settings.confirm(keywords): return self.get_keywords()
        self.keywords = keywords
        return self.keywords

    def get_performers(self):
        """
        Gets the performers value if not none else sets it from args or prompts.

        Returns
        -------
        list
            The performer usernames as strings in a list

        """

        if str(self.performers) == "unset": return []
        if len(self.performers) > 0: return self.performers
        # retrieve from args and return if exists
        performers = Settings.get_performers() or []
        # ensures all performers from files are included
        if len(self.files) > 0:
            for file in self.files:
                if hasattr(file, "performer"):
                    performer = getattr(file, "performer")
                    if performer not in performers:
                        performers.append(performer)
        if len(performers) > 0: 
            self.performers = performers
            return performers
        # prompt skip
        if not Settings.prompt("performers"):
            self.performers = "unset"
            return []
        question = {
            'type': 'input',
            'name': 'performers',
            'message': 'Performers:',
            'validate': ListValidator
        }
        performers = prompt(question)["performers"]
        performers = [n.strip() for n in performers.split(",")]
        # confirm performers
        if not Settings.confirm(performers): return self.get_performers()
        self.performers = performers
        return self.performers

    @staticmethod
    def is_tip(text):
        """
        Checks if the text contains a tip amount.

        Parameters
        ----------
        text : str
            The text to parse

        Returns
        -------
        bool
            Whether the text contains a tip or not
        int
            The tip amount contained, default 0

        """

        amount = 0
        if re.search(r'I sent you a \$[0-9]*\.00 tip ♥', text):
            amount = re.match(r'I sent you a \$([0-9]*)\.00 tip ♥', text).group(1)
            Settings.maybe_print("successfully found tip")
            Settings.dev_print("amount: {}".format(amount))
            return True, int(amount)
        elif re.search(r"I\'ve contributed \$[0-9]*\.00 to your Campaign", text):
            amount = re.match(r'I\'ve contributed \$([0-9]*)\.00 to your Campaign', text).group(1)
            Settings.maybe_print("successfully found campaign donation")
            Settings.dev_print("amount: {}".format(amount))
            return True, int(amount)
        return False, int(amount)

    def get_files(self):
        """
        Gets files from args specified source or prompts as necessary.

        Uses appropriate file select method as specified by runtime args:
        - google
        - remote
        - local

        Returns
        -------
        list
            Files in a list

        """

        if str(self.files) == "unset": return []
        if len(self.files) > 0: return self.files[:int(Settings.get_upload_max())]
        if len(Settings.get_input_as_files()) > 0:
            self.files = Settings.get_input_as_files()
            return self.files
        # prompt skip
        if not Settings.is_prompt() and Settings.get_category() == None:
            self.files = "unset"
            return []
        files = []
        if len(self.files) == 0:
            files = File.select_file_upload_method()
            if str(files[0]) == "unset" or str(files) == "unset":
                self.files = "unset"
                files = []
                if Settings.is_prompt(): return []
        if files == None: files = []
        # get files from appropriate source's menu selection
        if Settings.get_source() == "google":
            googleFiles = Google_File.get_files()
            if len(files) == 0 and len(googleFiles) > 0:
                files = Google_File.select_files()
            elif len(files) == 0 and len(googleFiles) == 0:
                self.files = "unset"
                return []
        elif Settings.get_source() == "remote":
            remoteFiles = Remote.get_files()
            if len(remoteFiles) > 0:
                files = Remote.select_files()
            elif len(files) == 0 and len(remoteFiles) == 0:
                self.files = "unset"
                return []
        elif Settings.get_source() == "local":
            localFiles = File.get_files()
            if len(files) == 0 and len(localFiles) > 0:
                files = File.select_files()
            elif len(files) == 0 and len(localFiles) == 0:
                self.files = "unset"
                return []
        filed = []
        for file in files:
            # turn all folders into their files
            if isinstance(file, Folder) or isinstance(file, Google_Folder): filed.extend(file.get_files())
            else:
                # flag that the files include a performer
                if hasattr(file, "performer"):
                    self.hasPerformers = True
                filed.append(file)
        self.files = filed[:int(Settings.get_upload_max())]
        return self.files

    def get_expiration(self):
        """
        Gets the expiration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The expiration as an int

        """

        if str(self.expiration) == "unset": return None
        if self.expiration: return self.expiration
        # retrieve from args and return if exists
        expires = Settings.get_expiration() or None
        if expires: 
            self.expiration = expires
            return expires
        # prompt skip
        if not Settings.prompt("expiration"):
            self.expiration = "unset"
            return None
        question = {
            'type': 'input',
            'name': 'expiration',
            'message': 'Expiration [1, 3, 7, 99 (\'No Limit\')]',
            'validate': ExpirationValidator
        }
        expiration = prompt(question)["expiration"]
        # confirm expiration
        if not Settings.confirm(expiration): return self.get_expiration()
        self.expiration = expiration
        return self.expiration

    def get_poll(self):
        """
        Gets the poll value if not none else sets it from args or prompts.

        Returns
        -------
        Poll
            Poll object with proper values

        """

        if str(self.poll) == "unset": return None
        # check if poll is ready
        if self.poll and self.poll.check(): return self.poll
        # prompt skip
        if Settings.is_prompt() and not Settings.prompt("poll"):
            self.poll = "unset"
            return None
        poll = Poll()
        # ensure the poll has non default values
        poll.get()
        # check if valid poll
        if not poll.check(): return None
        self.poll = poll
        return poll

    def get_price(self):
        """
        Gets the price value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The price as an int

        """

        if self.price: return self.price
        # retrieve from args and return if exists
        price = Settings.get_price() or None
        if price: 
            self.price = price
            return price
        if not Settings.prompt("price"): return ""
        question = {
            'type': 'input',
            'name': 'price',
            'message': 'Price',
            'validate': PriceValidator,
            'filter': lambda val: int(val)
        }
        price = prompt(question)["price"]
        if not Settings.confirm(price): return self.get_price()
        self.price = price
        return self.price

    def get_recipients(self):
        """
        Gets the recipients value if not none else sets it from args or prompts.

        Returns
        -------
        list
            Usernames in a list

        """

        if len(self.users) > 0: return self.users
        # if no recipients, prompt for them
        if len(self.recipients) == 0 and len(Settings.get_users()) > 0: 
            self.users = Settings.get_users()
        elif len(self.recipients) == 0 and Settings.get_user(): 
            self.users = [Settings.get_user()]
        # select users
        elif len(self.recipients) == 0:
            self.users = User.select_users()
        return self.users

    def get_schedule(self):
        """
        Gets the schedule value if not none else sets it from args or prompts.

        Returns
        -------
        Schedule
            Schedule object with proper values

        """

        if str(self.schedule) == "unset": return None
        if self.schedule: return self.schedule
        # prompt skip
        if Settings.is_prompt() and not Settings.prompt("schedule"):
            self.schedule = "unset"
            return None
        schedule = Schedule()
        schedule.get()
        # checks if schedule is valid
        if not schedule.check(): return None
        self.schedule = schedule
        return schedule
        
    def get_text(self):
        """
        Gets the text value if not none else sets it from args or prompts.

        Returns
        -------
        str
            The text to enter

        """

        if self.text: return self.text
        # retrieve from args and return if exists
        text = Settings.get_text() or None
        if text: 
            self.text = text
            return text
        # prompt skip
        if not Settings.prompt("text"): return None
        question = {
            'type': 'input',
            'name': 'text',
            'message': 'Text:'
        }
        text = prompt(question)["text"]
        # confirm text
        if not Settings.confirm(text): return self.get_text()
        self.text = text
        return self.text

    def get(self):
        """Gets all values"""

        if self.gotten: return
        self.get_text()
        self.get_keywords()
        self.get_price()
        self.get_poll()
        self.get_schedule()
        self.get_expiration()
        self.get_files()
        self.get_recipients()
        self.set_text()
        if Settings.get_performer_category() or self.hasPerformers:
            self.get_performers()
        else: # might as well skip asking if not pulling from performer category
            self.performers = "unset"
        self.gotten = True

    def get_post(self):
        """Gets values to populate as a post"""

        if self.gotten: return
        self.get_text()
        self.get_keywords()
        self.get_poll()
        self.get_schedule()
        self.get_expiration()
        self.get_files()
        self.set_text()
        if Settings.get_performer_category() or self.hasPerformers:
            self.get_performers()
        else:
            self.performers = "unset"
        self.gotten = True

    def get_message(self):
        """Gets values to populate as a message"""

        if self.gotten: return
        self.get_recipients()
        self.get_text()
        self.get_price()
        self.get_files()
        self.set_text()
        if Settings.get_performer_category() or self.hasPerformers:
            self.get_performers()
        else:
            self.performers = "unset"
        self.gotten = True

    def set_keywords(self):
        """Sets keywords from this object's file's title"""

        if len(self.get_keywords()) == 0 and len(self.get_files()) > 0:
            self.keywords = self.files[0].get_parent()["title"].split(" ")
            for keyword in self.keywords:
                if str(keyword) in str(self.text):
                    self.keywords = []
            
    def set_text(self):
        """Sets text from this object's file's title"""

        if not self.text and len(self.get_files()) > 0:
            self.text = self.files[0].get_title()
            # if "_" in str(self.text):
            if re.match("[0-9]_[0-9]", self.text) is not None:
                self.text = self.files[0].get_parent()["title"]
            else:
                try: 
                    int(self.text)
                    # is a simple int
                    if int(self.text) > 20:
                        self.text = self.files[0].get_parent()["title"]
                except Exception as e:
                    # not a simple int
                    # do nothing cause probably set already
                    pass
            self.text = self.text.replace("_", " ")
            self.set_keywords()

########################################################################################
########################################################################################
########################################################################################

class Poll:
    """OnlyFans Poll class"""

    def __init__(self):
        """OnlyFans Poll object"""

        # duration of poll
        self.duration = None
        # list of strings
        self.questions = []
        # prevents double prompts
        self.gotten = False

    def check(self):
        """Check if poll is ready with valid values

        Returns
        -------
        bool
            Whether the poll is ready to be posted or not

        """

        if len(self.get_questions()) > 0: return True
        if self.get_duration(): return True
        return False

    def get(self):
        if self.gotten: return
        gotten = self.get_duration()
        # return early if skipped
        if not gotten: return
        gotten = self.get_questions()
        # return early if skipped
        if not gotten: return
        self.gotten = True

    def get_duration(self):
        """
        Gets the duration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The duration as an int

        """

        if self.duration: return self.duration
        # retrieve from args and return if exists
        duration = Settings.get_duration()
        if duration: 
            self.duration = duration
            return duration
        # prompt skip
        if not Settings.prompt("duration"): return None
        question = {
            'type': 'input',
            'name': 'duration',
            'message': 'Duration [1, 3, 7, 99 (\'No Limit\')]',
            'validate': DurationValidator
        }
        duration = prompt(question)["duration"]
        # confirm duration
        if not Settings.confirm(duration): return self.get_duration()
        self.duration = duration
        return self.duration

    def get_questions(self):
        """
        Gets the questions value if not none else sets it from args or prompts.

        Returns
        -------
        list
            The questions as strings in a list

        """

        if len(self.questions) > 0: return self.questions
        # retrieve from args and return if exists
        questions = Settings.get_questions()
        if len(questions) > 0: 
            self.questions = questions
            return questions
        # prompt skip
        if not Settings.prompt("questions"): return []
        print("Enter Questions")
        while True:
            question = {
                'type': 'input',
                'name': 'question',
                'message': 'Question:',
            }
            answers = prompt(question)["question"]
            if str(question) == "": break
            questions.append(question)
        # confirm questions
        if not Settings.confirm(questions): return self.get_questions()
        self.questions = questions
        return self.questions

########################################################################################
########################################################################################
########################################################################################

class Promotion:
    """Promotion class"""

    def __init__(self):
        """Promotion object"""

        # the amount to discount
        self.amount = None
        # the number of trials to allow
        self.limit = None
        # the expiration of the trial
        self.expiration = None
        # the duration of the discount
        self.duration = None
        # the user to apply the promotion to
        self.user = None
        # the message to provide with the promotion
        self.message = None
        # prevents double prompts
        self.gotten = False

    @staticmethod
    def apply_to_user():
        """Applies promotion directly to user via their profile page
        
           Applying a discount to a user requires:
           - amount
           - duration
           - expiration
           - message
           - user

        """

        print("Promotion - Apply To User")
        p = Promotion()
        # ensure the promotion has non default values, return early if missing
        # p.get()
        gotten = p.get_amount()
        if not gotten: return
        gotten = p.get_duration()
        if not gotten: return
        gotten = p.get_expiration()
        if not gotten: return
        gotten = p.get_message()
        if not gotten: return
        gotten = p.get_user()
        if not gotten: return
        # prompt skip
        if Settings.is_prompt():
            if not Settings.prompt("Promotion"): return
        from .driver import Driver
        # get default driver and apply the promotion directly
        Driver.get_driver().promotion_user_directly(promotion=p)

    @staticmethod
    def create_campaign():
        """Creates a Promotional Campaign

           A campaign consists of:
           - amount
           - duration
           - expiration
           - limit
           - user
           - text

        """

        print("Promotion - Creating Campaign")
        p = Promotion()
        # ensure the promotion has non default values, return early if missing
        # p.get()
        gotten = p.get_amount()
        if not gotten: return
        gotten = p.get_user()
        if not gotten: return
        gotten = p.get_expiration()
        if not gotten: return
        gotten = p.get_limit()
        if not gotten: return
        gotten = p.get_duration()
        if not gotten: return
        gotten = p.get_message()
        if not gotten: return
        # prompt skip
        if Settings.is_prompt():
            if not Settings.prompt("Promotion"): return
        from .driver import Driver
        # get the default driver and enter the promotion campaign
        Driver.get_driver().promotional_campaign(promotion=p)

    # requires the copy/paste and email steps
    @staticmethod
    def create_trial_link():
        """Creates a Promotional Trial Link

           A trial link consists of:
           - duration
           - expiration
           - limit
           - message
           - user
            
           Note: this creates a free trial link but does NOT send it to the user
           because it is incomplete. The copy/paste step to message to a user is nonfunctioning.           

        """

        print("Promotion - Creating Trial Link")
        p = Promotion()
        # ensure the promotion has non default values, return early if missing
        # p.get()
        gotten = p.get_duration()
        if not gotten: return
        gotten = p.get_expiration()
        if not gotten: return
        gotten = p.get_limit()
        if not gotten: return
        gotten = p.get_message()
        if not gotten: return
        gotten = p.get_user()
        if not gotten: return
        # if not self.gotten: return
        if Settings.is_prompt():
            if not Settings.prompt("Promotion"): return
        # limit, expiration, months, user
        from .driver import Driver
        link = Driver.get_driver().promotional_trial_link(promotion=p)
        # text = "Here's your free trial link!\n"+link
        # Settings.dev_print("Link: "+str(text))
        # Settings.send_email(email, text)

    def get(self):
        """Update the promotion object's default values"""

        if self.gotten: return
        gotten = self.get_user()
        gotten = self.get_amount()
        gotten = self.get_expiration()
        gotten = self.get_limit()
        gotten = self.get_duration()
        gotten = self.get_message()
        self.gotten = True

    def get_amount(self):
        """
        Gets the amount value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The amount as an int

        """

        if self.amount: return self.amount
        # retrieve from args and return if exists
        amount = Settings.get_amount() or None
        if amount: 
            self.amount = amount
            return amount
        # prompt skip
        if not Settings.prompt("amount"): return None
        question = {
            'type': 'input',
            'name': 'amount',
            'message': 'Amount:',
            'validate': AmountValidator,
            'filter': lambda val: int(myround(int(val)))
        }
        amount = prompt(question)["amount"]
        # confirm amount
        if not Settings.confirm(amount): return self.get_amount()
        self.amount = amount
        return self.amount

    def get_expiration(self):
        """
        Gets the expiration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The expiration as an int

        """

        if self.expiration: return self.expiration
        # retrieve from args and return if exists
        expiration = Settings.get_expiration() or None
        if expiration: 
            self.expiration = expiration
            return expiration
        # prompt skip
        if not Settings.prompt("expiration"): return None
        question = {
            'type': 'input',
            'name': 'expiration',
            'message': 'Expiration [1, 3, 7, 99 (\'No Limit\')]',
            'validate': ExpirationValidator
        }
        expiration = prompt(question)["expiration"]
        # confirm expiration
        if not Settings.confirm(expiration): return self.get_expiration()
        self.expiration = expiration
        return self.expiration

    def get_limit(self):
        """
        Gets the expiration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The expiration as an int

        """

        if self.limit: return self.limit
        # retrieve from args and return if exists
        limit = Settings.get_limit() or None
        if limit: 
            self.limit = limit
            return limit
        # prompt skip
        if not Settings.prompt("limit"): return None
        question = {
            'type': 'input',
            'name': 'limit',
            'message': 'Limit (in days or months)',
            'validate': LimitValidator
        }
        limit = prompt(question)["limit"]
        # confirm limit
        if not Settings.confirm(limit): return self.get_limit()
        self.limit = limit
        return self.limit

    def get_message(self):
        """
        Gets the message value if not none else sets it from args or prompts.

        Returns
        -------
        str
            The message as a str

        """

        if self.message != None: return self.message
        # retrieve from args and return if exists
        message = Settings.get_text() or None
        if message: 
            self.message = message
            return message
        # prompt skip
        if not Settings.prompt("message"): return ""
        question = {
            'type': 'input',
            'name': 'message',
            'message': 'Message:'
        }
        message = prompt(question)["message"]
        # confirm message
        if not Settings.confirm(message): return self.get_text()
        self.message = message
        return self.message

    def get_duration(self):
        """
        Gets the duration value if not none else sets it from args or prompts.

        Returns
        -------
        int
            The duration as an int

        """

        if self.duration: return self.duration
        # retrieve from args and return if exists
        duration = Settings.get_promo_duration() or None
        if duration: 
            self.duration = duration
            return duration
        # duration skip
        if not Settings.prompt("duration"): return None
        question = {
            'type': 'input',
            'name': 'duration',
            'message': 'Duration [1 day, 3 days, 7 days, ...]',
            'validate': PromoDurationValidator
        }
        duration = prompt(question)["duration"]
        # confirm duration
        if not Settings.confirm(duration): return self.get_duration()
        self.duration = duration
        return self.duration

    def get_user(self):
        """
        Populate and get the username value

        If not found in args and prompt is enabled, ask for value.

        Returns
        -------
        User
            the user to apply the promotion to

        """

        if self.user: return self.user
        user = User.select_user()
        self.user = user.username
        return self.user

    @staticmethod
    def grandfathered():
        """
        Executes the 'Grandfather' promotion model

        In groups of 5, existing users will be added to the 'Grandfathered' OnlyFans list and
        then provided with the max discount for the max months. If the process interrupts, 
        running again will continue to discount users not yet added to the list.

        """

        print("Promotion - Grandfather")
        # prompt skip
        if Settings.is_prompt():
            if not Settings.prompt("Grandfather"): return
        Settings.maybe_print("getting users to grandfather")
        # get all users
        users = User.get_all_users()
        from .driver import Driver
        # get all users from logged in user's 'grandfathered' list
        users_, name, number = Driver.get_driver().get_list(name="grandfathered")
        # remove all users that have already been grandfathered from the list of all users
        # users = [user for user in users if user not in users_] # i guess doesn't work?
        for i, user in enumerate(users[:]):
            for user_ in users_:
                for key, value in user_.items():
                    if str(key) == "username" and str(user.username) == str(value):
                        users.remove(user)

        def chunks(lst, n):
            """Yield successive n-sized chunks from lst."""
            for i in range(0, len(lst), n):
                yield lst[i:i + n]

        # get users in groups of 5 to allow performance over interrupts
        userChunks = chunks(users, 5)
        num = 1
        for userChunk in userChunks:
            print("Chunk: {}/{}".format(num, len(users)/5))
            num += 1
            # add users to 'grandfathered' list prior to discounting
            Settings.maybe_print("grandfathering: {}".format(len(userChunk)))
            try:
                successful = Driver.get_driver().add_users_to_list(users=userChunk, number=number, name="grandfathered")
                # if successful then discount
                if not successful: return
                d = Discount() # discount will fill defaults with promotion values
                d.grandfatherer(users=userChunk)
            except Exception as e:
                Settings.dev_print(e)

    @staticmethod
    def menu():
        """Promotion menu interface"""

        if not Settings.is_debug():
            print("### Not Available ###")
            return
        action = Promotion.ask_action()
        if (action == 'Back'): pass
        elif (action == 'trial'): Promotion.create_trial_link()
        elif (action == 'campaign'): Promotion.create_campaign()
        elif (action == 'user'): Promotion.apply_to_user()
        elif (action == 'grandfather'): Promotion.grandfathered()

    @staticmethod
    def ask_action():
        """Promotion menu selection

        Returns
        -------
        str
            The menu action to take

        """

        # arg - promotion_method: campaign, trial, user, grandfather
        options = ["back", 
            "campaign", # 
            "grandfather" # this mostly completely works
            "trial", # this isn't even finished but it does mostly work
            "user", # should this be here?
        ]
        menu_prompt = {
            'type': 'list',
            'name': 'action',
            'message': 'Please select a promotion action:',
            'choices': [str(option).title() for option in options],
            'filter': lambda val: str(val).lower()
        }
        answers = prompt(menu_prompt)
        return answers['action']

########################################################################################
########################################################################################
########################################################################################

class Schedule:

    def __init__(self):
        self.date = None
        self.time = None
        ##
        self.hour = "00"
        self.minute = "00"
        self.year = "0"
        self.month = "0"
        self.day = "0"
        self.suffix = "am"
        ##
        self.gotten = False

    def apply(self):
        self.get()
        if not self.gotten: return
        if not Settings.prompt("Schedule"): return

    def check(self):
        if self.get_date() and self.get_time(): return True
        return False

    def get(self):
        if self.gotten: return
        if self.get_date():
            date = self.get_date()
            maybe_print(0)
            maybe_print(date)
            if self.get_time():
                maybe_print(11)
                maybe_print(self.get_time())
                if "00:00:00" in str(date):
                    date = str(date).replace("00:00:00", self.get_time())
                else:
                    date = "{} {}".format(date, self.get_time())
            if "am" in self.get_time().lower(): self.suffix = "am"
            elif "pm" in self.get_time().lower(): self.suffix = "pm"
            maybe_print(1)
            maybe_print(date)
            date = datetime.strptime(str(date), "%Y-%m-%d %H:%M %p")
            maybe_print(2)
            maybe_print(date)
            self.year = date.year
            self.month = date.strftime("%B")
            self.day = date.day
            self.hour = date.hour
            self.minute = date.minute
            maybe_print("year: {}".format(self.year))
            maybe_print("month: {}".format(self.month))
            maybe_print("day: {}".format(self.day))
            maybe_print("hour: {}".format(self.hour))
            maybe_print("minutes: {}".format(self.minute))
        self.gotten = True

    def get_date(self):
        """
        Gets the date value if not none else sets it from args or prompts.

        Returns
        -------
        str
            The date as a valid date string

        """

        if self.date: return self.date
        date = Settings.get_date() or None
        if date: 
            self.date = date
            return date
        # retrieve from args and return if exists
        schedule = Settings.get_schedule() or None
        if schedule:
            date = datetime.strptime(str(schedule), "%Y-%m-%d %H:%M:%S")
            self.date = date.date()
            return self.date
        # prompt skip
        if not Settings.prompt("date"): return None
        question = {
            'type': 'input',
            'name': 'date',
            'message': 'Enter a date (MM-DD-YYYY):',
            'validate': DateValidator
        }
        date = prompt(question)["date"]
        # confirm date
        if not Settings.confirm(date): return self.get_date()
        self.date = date
        return self.date

    def get_time(self):
        """
        Gets the time value if not none else sets it from args or prompts.

        Returns
        -------
        str
            The time as a valid time string

        """

        if self.time: return self.time
        # retrieve from args and return if exists
        time = Settings.get_time() or None
        if time: 
            time = datetime.strptime(str(time), "%Y-%m-%d %H:%M:%S")
            # Settings.dev_print(time)
            time = time.strftime("%I:%M %p")
            # Settings.dev_print(time)
            self.time = time
            return self.time
        # retrieve time from schedule args and return if exists
        schedule = Settings.get_schedule() or None
        if schedule:
            time = datetime.strptime(str(schedule), "%Y-%m-%d %H:%M:%S")
            # Settings.dev_print(time)
            time = time.strftime("%I:%M %p")
            # Settings.dev_print(time)
            self.time = time
            return self.time
        # prompt skip
        if not Settings.prompt("time"): return None
        question = {
            'type': 'input',
            'name': 'time',
            'message': 'Enter a time (HH:MM):',
            'validate': TimeValidator
        }
        time = prompt(question)["time"]
        # confirm time
        if not Settings.confirm(time): return self.get_time()
        self.time = time
        return self.time

# round to 5
def myround(x, base=5):
    return base * round(x/base)


import time
import threading
import concurrent.futures
from .driver import Driver
from .classes import Message
from .user import User
from .settings import Settings

REFRESH_DURATION = 60*9

# RUN_DURATION_ALL = 60*60
# RUN_DURATION_RECENT = 60*10

# commands = [
# 	"0) menu",
# 	# "1) dick pic""
# ]

# COMMANDS_AVAILABLE = "Commands available:\n0) menu\n1) notice me senpai"

MAX_BROWSERS = 1
# MAX_THREADS = 5

class Bot():

	USERS = []
	i = 0
	lock = threading.RLock()

	def __init__(self):
		self.driver = None
		self.drivers = []
		self.refreshing = None
		self.running = None
		self.lock = threading.RLock()
		self.locks = []

		##
		# self.refresher()

	@staticmethod
	def parse(user=None):
		print("Parsing: {} - {}".format(user.username, user.id))
		# check user for commands in unchecked messages
		# run command
		# print("user: {}".format(user))
		# print("parsing: {}".format(user.username))

		# if not user or not user.username or str(user) == "None" or str(user.username) == "None": return
		# commands = ["0) menu"]

		user.update_chat_log()

		unparsed = user.get_unparsed_messages()
		for message in unparsed:
			successful = False
			isTip, amount = Message.is_tip(message)
			if isTip:
				print(0)
				successful = Bot.tipped(user=user, amount=amount)
				print(1)
			# elif "0) menu" in str(message).lower():
			# 	successful = Bot.prompt(user=user)
			if successful:
				user.parse_message(message=message.message)
		Settings.dev_print("successfully parsed user: {} - {}".format(user.username, user.id))

	def parse_messages(self):
		# if self.running: self.running.stop()
		if not self.driver: self.driver = Driver(browser=None)

		# read all messages
		# users = Bot.USERS
		# if len(users) == 0:
		# 	users = User.get_all_users(driver=self.driver)
		# else:
		# 	users = User.get_recent_messagers(driver=self.driver)
		# 	# users = User.get_recent_messagers(notusers=users, driver=self.driver)
		# Bot.USERS = users

		users = User.get_recent_messagers(driver=self.driver)

		print("Users to parse: {}".format(len(users)))
		# self.running = threading.Timer(RUN_DURATION*len(users), self.run).start()
		# self.running = threading.Timer(RUN_DURATION, self.run).start()

		# respond to messages

		def threaded():
			def parse(user):
				self.lock.acquire()
				i = int(Bot.get_index())
				# if i > len(self.locks):
					# self.locks.append(threading.RLock())
				# self.locks[i].acquire()
				try:
					# self.lock.acquire()
					if not user.driver or not user.browser:
						if len(self.drivers) == 0:
							user.driver = self.driver
							self.drivers.append(user.driver)
							self.driver = None
						elif len(self.drivers) >= MAX_BROWSERS:
							user.driver = self.drivers[i]
						else:
							user.driver = Driver(browser=None)
							self.drivers.append(user.driver)
					self.lock.release()
					Bot.parse(user=user)
				except Exception as e:
					print(e)
					Settings.dev_print("failed to parse user: {} - {}".format(user.id, user.username))
				# finally:
					# self.locks[i].release()

			# for user in users:
			# 	prepare(user)

			# if "remote" in str(Settings.get_browser_type()): MAX_THREADS = 10
			with concurrent.futures.ThreadPoolExecutor(max_workers=MAX_BROWSERS) as executor:
				executor.map(parse, users)

		def single():
			for user in users:
				if not user.driver or not user.browser:
					# setattr(user, "driver", Driver(browser=None))
					# setattr(user, "browser", user.driver.spawn())
					setattr(user, "driver", self.driver)
				Bot.parse(user=user)

		if "remote" in Settings.get_browser_type() or "reconnect" in Settings.get_browser_type():
			single()
		else:
			threaded()

		time.sleep(60*10)
		if not self.driver: self.driver = Driver(browser=None)
		users = User.get_all_users(driver=self.driver)

		print("Users to parse: {}".format(len(users)))

		if "remote" in Settings.get_browser_type() or "reconnect" in Settings.get_browser_type():
			single()
		else:
			threaded()

		time.sleep(60*10)
		if not self.driver: self.driver = Driver(browser=None)
		users = User.get_recent_messagers(driver=self.driver)

		print("Users to parse: {}".format(len(users)))

		if "remote" in Settings.get_browser_type() or "reconnect" in Settings.get_browser_type():
			single()
		else:
			threaded()

		# self.parse_messages()

	@staticmethod
	def get_index():
		# Bot.lock.acquire()
		i = Bot.i
		Bot.i += 1
		if Bot.i == MAX_BROWSERS: Bot.i = 0
		# Bot.lock.release()
		return i

	@staticmethod
	def prompt(user=None):
		# show list of commands available
		user.message(message=COMMANDS_AVAILABLE)
		return True

	# refresh the Driver
	def refresh(self):
		self.driver.refresh()

	# handle the timer for refreshing the Driver
	def refresher(self):
		if not Settings.is_keep(): return
		if self.refreshing: self.refreshing.stop()
		self.refreshing = threading.Timer(REFRESH_DURATION, self.refresh).start()



	@staticmethod
	def tipped(user=None, amount=None):
		# for every $x amountsend 1 dick pic
		num = int(amount)%5
		Settings.dev_print("tipped num: {}".format(num))
		return user.send_dick_pics(num)


#!/usr/bin/python3

import logging
logging.getLogger('googleapicliet.discovery_cache').setLevel(logging.ERROR)

import os
import shutil
import datetime
import json
import sys
import subprocess
import pathlib
import io
from subprocess import PIPE, Popen
from pydrive.auth import GoogleAuth
from pydrive.drive import GoogleDrive
# from moviepy.editor import VideoFileClip
from apiclient.discovery import build
from httplib2 import Http
from oauth2client import file, client, tools
from apiclient.http import MediaFileUpload,MediaIoBaseDownload
##
from .settings import Settings

###################
##### Globals #####
###################

AUTH = False
CACHE = []
DRIVE = None
PYDRIVE = None
ONE_GIGABYTE = 1000000000
ONE_MEGABYTE = 1000000
FIFTY_MEGABYTES = 50000000
ONE_HUNDRED_KILOBYTES = 100000
OnlyFansFolder_ = None

# Video MimeTypes
# Flash   .flv    video/x-flv
# MPEG-4  .mp4    video/mp4
# iPhone Index    .m3u8   application/x-mpegURL
# iPhone Segment  .ts     video/MP2T
# 3GP Mobile  .3gp    video/3gpp
# QuickTime   .mov    video/quicktime
# A/V Interleave  .avi    video/x-msvideo
# Windows Media   .wmv    video/x-ms-wmv
MIMETYPES_IMAGES = "(mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"
MIMETYPES_VIDEOS = "(mimeType contains 'video/mp4' or mimeType contains 'video/quicktime' or mimeType contains 'video/x-ms-wmv' or mimeType contains 'video/x-flv')"
MIMETYPES_ALL = "(mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png' or mimeType contains 'video/mp4' or mimeType contains 'video/quicktime')"
MIMETYPE_FOLDER = "mimeType contains 'application/vnd.google-apps.folder'"

def print_same_line(text):
    sys.stdout.write('\r')
    sys.stdout.flush()
    sys.stdout.write(text)
    sys.stdout.flush()

################
##### Auth #####
################

def authGoogle():
    """Authorizes Google Drive API"""

    Settings.dev_print('Authenticating Google')
    try:
        # PyDrive
        gauth = GoogleAuth()
        if os.path.exists("/opt/onlysnarf/settings.yaml"):
            gauth = GoogleAuth(settings_file="/opt/onlysnarf/settings.yaml")
        # Try to load saved client credentials
        gauth.LoadCredentialsFile(Settings.get_google_path())
        Settings.dev_print('Loaded: Google Credentials')
        if gauth.credentials is None:
            # Authenticate if they're not there
            gauth.LocalWebserverAuth()
        elif gauth.access_token_expired:
            # Refresh them if expired
            gauth.Refresh()
        else:
            # Initialize the saved creds
            gauth.Authorize()
        # Save the current credentials to a file
        gauth.SaveCredentialsFile(Settings.get_google_path())
        global PYDRIVE
        PYDRIVE = GoogleDrive(gauth)
        # Drive v3 API (alternative downloads)
        SCOPES = 'https://www.googleapis.com/auth/drive'
        store = file.Storage(Settings.get_google_path())
        creds = store.get()
        if not creds or creds.invalid:
            flow = client.flow_from_clientsecrets(Settings.get_secret_path(), SCOPES)
            creds = tools.run_flow(flow, store)
        global DRIVE
        DRIVE = build('drive', 'v3', http=creds.authorize(Http()))
    except Exception as e:
        Settings.dev_print(e)
        print('Error: Unable to Authenticate w/ Google')
        return False
    Settings.dev_print('Authentication Successful') 
    return True

def checkAuth():
    """Check if Google Drive is authorized, if not then authorize"""

    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    return AUTH

###########################################
##### Archiving / Creating / Deleting #####
###########################################

def backup_file(file):
    """
    Backs up file to Google Drive in OnlyFans/posted folder

    Parameters
    ----------
    file : Google File
        Google Drive file to backup

    """

    try:
        global PYDRIVE
        # get backup folder
        backupTo = get_folder_by_name("posted")
        # determine category of folder to backup into
        stri = "posted/{}".format(backupTo["title"])
        if file.category or Settings.get_category():
            category = file.category or Settings.get_category()
            # get the folder to back up to
            backupTo = get_posted_folder_by_name(category)
            # if performer, get the proper inner category folder
            if str(category) == "performers" and Settings.get_category_performer():
                backupTo = get_folder_by_name(Settings.get_category_performer(), parent=backupTo)
            stri = "posted/{}".format(backupTo["title"])

        # check posted for folder to backup with existing name
        file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and title='{}'".format(backupTo['id'], str(file.get_parent()["title"]))}).GetList()
        if len(file_list) == 0:
            parentFolder = PYDRIVE.CreateFile({'title':str(file.get_parent()["title"]), 'parents':[{"kind": "drive#fileLink", "id": str(backupTo['id'])}], 'mimeType':'application/vnd.google-apps.folder'})
        else:
            parentFolder = file_list[0]

        parentFolder.Upload()
        Settings.dev_print("Moving To: {}".format(stri))
        # change parents of file to "move" it
        file.get_file()['parents'] = [{"kind": "drive#fileLink", "id": str(parentFolder['id'])}]
        file.get_file().Upload()
        print("File Backed Up: {}".format(file.get_title()))
    except Exception as e:
        Settings.dev_print(e)

def create_folders():
    """Create OnlySnarf category folders"""

    auth = checkAuth()
    if not auth: return
    print("Creating Folders: {}".format(Settings.get_drive_path()))
    # get root OnlySnarf folder in Drive
    OnlyFansFolder = get_folder_root()
    if OnlyFansFolder is None:
        print("Error: Unable To Create Category Folders")
        return
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(OnlyFansFolder['id'])}).GetList()
    # create each missing folder
    for folder in Settings.get_categories():
        found = False
        for folder_ in file_list:
            if str(folder) == folder_['title']:
                Settings.maybe_print("Found Folder: {}".format(folder))
                found = True
        if not found:
            if not Settings.is_create_missing():
                Settings.maybe_print("Skipping: Create Missing Category Folder - {}".format(folder))
                continue   
            Settings.maybe_print("Created Folder: {}".format(folder))
            contentFolder = PYDRIVE.CreateFile({"title": str(folder), "parents": [{"id": OnlyFansFolder['id']}], "mimeType": "application/vnd.google-apps.folder"})
            contentFolder.Upload()

def delete_folder(folder):
    """
    Delete folder

    Parameters
    ----------
    folder : Google Folder
        Google Drive folder to delete

    """

    if Settings.is_delete():
        try:
            print("Deleting folder: {}".format(folder["title"]))
            folder.Trash()
        except Exception as e: Settings.dev_print(e)

#################
##### Cache #####
#################

def cache_add(folders):
    """Add folder to local cache"""

    global CACHE
    for folder in folders:
        CACHE.append([folder['title'], folder])

def cache_check(folderName):
    """Check local cache for folder by name"""

    global CACHE
    for folder in CACHE:
        if str(CACHE[0]) == str(folderName):
            return CACHE[1]
    return False

####################
##### Download #####
####################

def download_file(file):
    """
    Download file using one of two methods

    1) PyDrive
    2) Google Drive - shows download %

    Parameters
    ----------
    file : Google File
        Google file to download

    Returns
    -------
    bool
        Whether or not the download was successful

    """

    Settings.maybe_print("Downloading File: {}".format(file.get_title()))
    def method_one():
        try:
            with open(str(file.get_path()), 'w+b') as output:
                # print("8",end="",flush=True)
                request = DRIVE.files().get_media(fileId=file.get_id())
                downloader = MediaIoBaseDownload(output, request)
                # print("=",end="",flush=True)
                done = False
                while done is False:
                    # print("=",end="",flush=True)
                    status, done = downloader.next_chunk()
                    if int(Settings.get_verbosity()) >= 1:
                        print("Downloading: %d%%\r" % (status.progress() * 100), end="")
                # print("D")
                Settings.maybe_print("Download Complete (1)")
        except Exception as e:
            Settings.dev_print(e)
            return False
        return True 
    def method_two():
        try:
            file.get_file().GetContentFile(file.get_path())
            Settings.maybe_print("Download Complete (2)")
        except Exception as e:
            Settings.dev_print(e)
            return False
        return True
    successful = method_one() or method_two()
    return successful

###############
##### Get #####
###############

def get_files_by_folder_id(folderID):
    """Get files of folder by id

    Parameters
    ----------
    folderID : str
        The folder id of the Google Folder to get the files of

    Returns
    -------
    list
        A list of Google Files in the matching folder

    """

    if not folderID:
        print("Error: Missing Folder ID")
        return
    auth = checkAuth()
    if not auth: return []
    return PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(folderID)}).GetList()
    
def get_file(id_):
    """Get file by id

    Parameters
    ----------
    id_ : str
        The id of the Google File to locate

    Returns
    -------
    Google File
        The located Google File

    """

    auth = checkAuth()
    if not auth: return
    myfile = PYDRIVE.CreateFile({'id': id_})
    # myfile.FetchMetadata()
    return myfile

def get_file_parent(id_):
    """
    Get parent file of file

    Parameters
    ----------
    id_ : str
        The id of the Google File to locate

    Returns
    -------
    Google Folder
        The located parent Google Folder

    """
    # auth = checkAuth()
    # if not auth: return
    # Settings.dev_print("getting file parent: {}".format(id_))
    parent = get_file(id_)["parents"][0]
    parent = PYDRIVE.CreateFile({'id': parent["id"]})
    return parent


def get_folder_by_name(folderName, parent=None):
    """
    Find folder by name with parent
    
    Parameters
    ----------
    folderName : str
        Name of folder to find
    parent : str
        Optional parent folder to search within 

    Returns
    -------
    Google Folder
        The located / created Google Folder

    """

    global CACHE
    if cache_check(folderName): return cache_check(folderName)
    auth = checkAuth()
    if not auth: return
    if str(folderName) in str(Settings.get_categories()) and not parent:
        parent = get_folder_root()
    if not parent: parent = get_folder_root()
    Settings.maybe_print("Getting Folder: {}".format(folderName))
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(parent['id'])}).GetList()
    for folder in file_list:
        if str(folder['title'])==str(folderName):
            Settings.maybe_print("Found Folder: {}".format(folderName))
            cache_add([folder])
            return folder
    if not Settings.is_create_missing():
        Settings.maybe_print("Skipping: Create Missing Folder - {}".format(folderName))
        return None
    # create if missing
    folder = PYDRIVE.CreateFile({"title": str(folderName), "mimeType": "application/vnd.google-apps.folder", "parents": [{"kind": "drive#fileLink", "id": parent["id"]}]})
    folder.Upload()
    Settings.maybe_print("Created Folder: {}".format(folderName))
    return folder

def get_images_of_folder(folder):
    """
    Get all image files of folder

    Parameters
    ----------
    folder : Google Folder
        The Google Folder to search for images

    Returns
    -------
    list
        The images located within the folder

    """

    image_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and {}".format(folder['id'], MIMETYPES_IMAGES)}).GetList()
    if len(image_list) > 0:
        Settings.dev_print('Images: {}'.format(len(image_list)))
    else:
        Settings.maybe_print("Images Folder (empty): {}".format(folder['title']))
        if Settings.is_delete_empty(): delete_folder(folder)
    return image_list

def get_videos_of_folder(folder):
    """
    Gets all video files of folder

    Parameters
    ----------
    folder : Google Folder
        The Google Folder to search for videos

    Returns
    -------
    list
        The videos located within the folder

    """

    video_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and {}".format(folder['id'], MIMETYPES_VIDEOS)}).GetList()
    if len(video_list) > 0:
        Settings.dev_print('Videos: {}'.format(len(video_list)))
    else:
        Settings.maybe_print("Video Folder (empty): {}".format(folder['title']))
        if Settings.is_delete_empty(): delete_folder(folder)
    return video_list

# returns first layer of files found
def get_folders_of_folder_by_keywords(folder):
    """
    Gets all folders in folder by the matching keywords

    Parameters
    ----------
    folder : Google Folder
        The Google Folder to search for keywords within

    Returns
    -------
    list
        The matching keyword folders located within folder

    """

    auth = checkAuth()
    if not auth: return []
    Settings.maybe_print("Getting Keywords in: {}".format(folder['title']))
    foundFolders = []
    folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and {}".format(folder['id'], MIMETYPE_FOLDER)}).GetList()
    for folder in folders:
        if Settings.get_drive_keyword() and str(folder['title']) != str(Settings.get_drive_keyword()):
            Settings.dev_print('{} -> not keyword'.format(folder['title']))
            continue
        elif Settings.get_drive_ignore() and str(folder['title']) == str(Settings.get_drive_ignore()):
            Settings.dev_print('{} -> by not keyword'.format(folder['title']))
            continue
        elif str(folder['title']) == str(Settings.get_drive_keyword):
            Settings.dev_print('{} -> by keyword'.format(folder['title']))
        else:
            Settings.dev_print("{}".format(folder['title']))
        foundFolders.append(folder)
    if Settings.get_sort_method() == "ordered":
        foundFolders = sorted(foundFolders, key = lambda x: x["title"])
    return foundFolders

def get_posted_folder_by_name(folderName, parent=None):
    """
    Get folder in "posted" folders by name.

    Parameters
    ----------
    folderName : str
        Name of the folder to find
    parent : Google Folder
        Optional Google Folder to search in

    Returns
    -------
    Google Folder
        The Google Folder that matches the folderName

    """

    global CACHE
    if cache_check(folderName): return cache_check(folderName)
    auth = checkAuth()
    if not auth: return
    Settings.maybe_print("Getting Posted Folder: {}".format(folderName))
    if parent is None:
        parent = get_folder_root()
    posted = None
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(parent['id'])}).GetList()
    for folder in file_list:
        if str(folder['title'])=="posted":
            Settings.maybe_print("Found Folder: posted")
            cache_add([folder])
            posted = folder
    if posted == None:
        if not Settings.is_create_missing():
            Settings.maybe_print("Skipping: Create Missing Folder - {}".format("posted"))
            return None        
        # create if missing
        posted = PYDRIVE.CreateFile({"title": str("posted"), "mimeType": "application/vnd.google-apps.folder", "parents": [{"kind": "drive#fileLink", "id": parent}]})
        posted.Upload()
        Settings.maybe_print("Created Folder: {}".format("posted"))
    folder= None
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(posted['id'])}).GetList()
    for folder_ in file_list:
        if str(folder_['title'])==str(folderName):
            Settings.maybe_print("Found Folder: {}".format(folderName))
            cache_add([folder_])
            return folder_
    if not Settings.is_create_missing():
        Settings.maybe_print("Skipping: Create Missing Folder - {}".format(folderName))
        return None
    # create if missing
    folder = PYDRIVE.CreateFile({"title": str(folderName), "mimeType": "application/vnd.google-apps.folder", "parents": [{"kind": "drive#fileLink", "id": posted['id']}]})
    folder.Upload()
    Settings.maybe_print("Created Folder: {}".format(folderName))
    cache_add([folder])
    return folder

def get_folders_of_folder(folder):
    """
    Get folders of folder.

    Parameters
    ----------
    folder : Google Folder
        The folder to get the folders of

    Returns
    -------
    list
        A list of the folders found

    """

    global CACHE
    if cache_check(folder): return cache_check(folder)
    auth = checkAuth()
    if not auth: return []
    Settings.maybe_print("Getting Folders of: {}".format(folder['title']))
    folders = []
    folder_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and {}".format(folder['id'], MIMETYPE_FOLDER)}).GetList()
    for folder in folder_list:
        file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(folder['id'])}).GetList()
        if len(file_list) > 0:
            Settings.maybe_print("Found Folder: {}".format(folder['title']))
            folders.append(folder)
        else:
            Settings.maybe_print("Found Folder (empty): {}".format(folder['title']))
            if Settings.is_delete_empty(): delete_folder(folder)
    cache_add(folders)
    return folders

# Creates the OnlyFans folder structure
def get_folder_root():
    """
    Gets the OnlySnarf root folder.

    Creates the OnlySnarf folder structure if missing.

    Returns
    -------
    Google Folder
        The root OnlySnarf folder

    """

    auth = checkAuth()
    if not auth: return
    global OnlyFansFolder_
    if OnlyFansFolder_ is not None:
        return OnlyFansFolder_
    OnlyFansFolder = None
    if Settings.get_drive_path() != "":
        mount_root = "root"
        root_folders = str(Settings.get_drive_path()).split("/")
        Settings.maybe_print("Mount Folders: {}".format("/".join(root_folders)))    
        for folder in root_folders:
            mount_root = get_folder_by_name(mount_root, parent=folder)
            if mount_root is None:
                mount_root = "root"
                print("Warning: Drive Mount Folder Not Found")
                break
        mount_root = get_folder_by_name(mount_root, parent=Settings.get_drive_root())
        if mount_root is None:
            mount_root = {"id":"root"}
            print("Warning: Drive Mount Folder Not Found")
        else:
            Settings.maybe_print("Found Root (alt): {}/{}".format(Settings.get_drive_path(), Settings.get_drive_root()))
        OnlyFansFolder = mount_root
    else:
        file_list = PYDRIVE.ListFile({'q': "'root' in parents and trashed=false"}).GetList()
        for folder in file_list:
            if str(folder['title']) == str(Settings.get_drive_path()):
                OnlyFansFolder = folder
                Settings.maybe_print("Found Root: {}".format(Settings.get_drive_path()))
    if OnlyFansFolder is None:
        print("Creating Root: {}".format(Settings.get_drive_path()))
        OnlyFansFolder = PYDRIVE.CreateFile({"title": str(Settings.get_drive_path()), "mimeType": "application/vnd.google-apps.folder"})
        OnlyFansFolder.Upload()
    OnlyFansFolder_ = OnlyFansFolder
    return OnlyFansFolder_

##################
##### Upload #####
##################

def upload_file(file=None):
    """
    Upload file to Google Drive

    Parameters
    ----------
    file : Google File
        Google file to be uploaded

    Returns
    -------
    bool
        Whether or not the upload was successful

    """

    auth = checkAuth()
    if not auth: return False
    if not file:
        print("Error: Missing File")
        return False
    if Settings.is_debug():
        print("Skipping Google Upload (debug): {}".format(filename))
        return False
    elif not Settings.is_backup():
        print('Skipping Google Upload (disabled): {}'.format(filename))
        return False
    else:
        print('Google Upload (file): {}'.format(filename))
    uploadedFile = PYDRIVE.CreateFile({'title':str(file.get_title()), 'parents':[{"kind": "drive#fileLink", "id": str(file.get_parent()["id"])}],'mimeType':str(file.get_mimetype())})
    uploadedFile.SetContentFile(file.get_path())
    uploadedFile.Upload()
    return True

def upload_gallery(files=[]):
    """
    Upload files to folder.

    Parameters
    ----------
    files : list
        The list of files to upload

    Returns
    -------
    bool
        Whether or not the upload was successful

    """

    parent = get_folder_by_name("posted")
    if not parent:
        print("Error: Missing Posted Folder")
        return False
    if Settings.is_debug():
        print("Skipping Google Upload (debug): {}".format(path))
        return False
    elif not Settings.is_backup():
        print('Skipping Google Upload (disabled): {}'.format(path))
        return False
    else:
        print('Google Upload: {}'.format(path))
    tmp_folder = PYDRIVE.CreateFile({'title':str(datetime.datetime.now()), 'parents':[{"kind": "drive#fileLink", "id": str(parent['id'])}],'mimeType':'application/vnd.google-apps.folder'})
    tmp_folder.Upload()
    successful = False
    for file in files:
        setattr(file, "parent", tmp_folder)
        successful = upload_file(file=file)
    return successful



#!/usr/bin/python3
# Profile Settings
import json
from PyInquirer import prompt
from .driver import Driver
from .settings import Settings
from .user import User

class Profile:
    TABS = ["profile", "advanced", "messaging", "notifications", "security", "story", "other"]

    # profile settings are either:
    #   enabled or disabled
    #   display text, variable type, variable name in settings
    def __init__(self):
        profile = Profile.fill_data()
        for key, value in profile.items():
            setattr(self, str(key), value)

    # Backup

    def ask_backup():
        menu_prompt = {
            'type': 'list',
            'name': 'action',
            'message': 'Please select a backup action:',
            'choices': ['Back', 'Content', 'Messages', 'Content & Messages'],
            'filter': lambda val: str(val).lower()
        }
        answers = prompt(menu_prompt)
        return answers['action']

    def backup_menu():
        if not Settings.is_debug():
            print("### Not Available ###")
            return
        backup = Profile.ask_backup()
        if (backup == 'back'): return Profile.menu()
        elif (backup == 'content'):
            Profile.backup_content()
        elif (backup == 'messages'):
            Profile.backup_messages()
        elif (backup == 'content & messages'):
            Profile.backup_content()
            Profile.backup_messages()
        Profile.menu()

    @staticmethod
    def backup_content():
        print("Backing Up: Content")
        Driver.download_content()
        print("Backed Up: Content")

    @staticmethod
    def backup_messages():
        print("Backing Up: Messages")
        # select user
        user = "all"
        if Settings.prompt("select user"):
            user = User.select_user()
        Driver.download_messages(user)
        print("Backed Up: Messages")

    # new - advertise - tweet to advertise new account, tweet to ask about what you should post
    def advertise():
        pass

    def advertise_menu():
        pass

    # check settings for 'profile completion'
    # |- subscription price |- calculate recommended price from percentile count, posts numbers etc 
    # |-  Reward for subscriber referrals
    # |- about, location, website url, wishlist
    # |- if connected twitter/google
    # |- welcome message enabled
    # |- two step authentication
    # |- watermark enabled & custom text
    def check():
        if not Settings.is_debug():
            print("### Not Available ###")
            return
        print("Checking Profile Settings")
        profile = Profile.read_local() or Profile.create()
        desiredProfile = {
            "subprice":"avalue", # do manually to check price number as int
            "about":"avalue",
            "location":"avalue",
            "websiteURL":"avalue",
            "wishlist":"avalue",
            "twitter":"avalue",
            "google":"avalue",
            "welcomeMessage":"avalue",
            "twoStepAuth":True,
            "watermark":True,
            "watermarkPhoto":True,
            "watermarkVideo":True
        }
        # get profile settings
        # check against preferred settings
        # output message
        failed = False
        for key, value in profile.items():
            for key_, value_ in desiredProfile.items():
                Settings.dev_print("{}: {} = {}".format(key, value, value_))
                if value and str(value_) != "avalue":
                    if value != value_:
                        print("Warning: Unrecommended setting - {}".format(key))
                        failed = True
                elif not value or str(value) != str(value_):
                    print("Warning: Unrecommended setting - {}".format(key))
                    failed = True
        if failed:
            print("Error: Profile check failed!")
            return False
        print("Success! Profile check completed.")
        return True

    # update basic new profile settings w/ profile settings or prompt 
    # get Twitter profile & banner and use to update profile & banner
    # About, Price, Wishlist
    # watermark enabled & custom text == username
    def setup():
        if not Settings.is_debug():
            print("### Not Available ###")
            return
        print("Setting up basic profile settings")
        profile = Profile.read_local() or Profile.create()
        desiredProfile = {
            "subprice":"avalue", # do manually to check price number as int
            "about":"avalue",

            "welcomeMessage":"avalue",
            "watermark":True,
            "watermarkText":True
        }
        
        # compare to existing values to ignore already correctly set values
        for key, value in profile.items():
            for key_, value_ in desiredProfile.items():
                if str(key) == "subprice":
                    # do stuff
                    continue

                Settings.dev_print("{}: {} = {}".format(key, value, value_))
                setattr(profile, str(key), value_)

        # search for twitter banner
        twitterBanner = None
        # search for twitter profile photo
        twitterProfile = None
        # update both
        setattr(profile, "coverImage", twitterBanner)
        setattr(profile, "profilePhoto", twitterProfile)
        Profile.sync_to_profile(profile=profile)

    def posts_menu():
        if not Settings.is_debug():
            print("### Not Available ###")
            return
        action = Profile.ask_new()
        if (action == 'back'): return Profile.menu()
        elif (action == 'advertise'):
            Profile.advertise()
        # elif (action == 'new')
        # elif (action == 'new')

        Profile.menu()

    def ask_new():
        menu_prompt = {
            'type': 'list',
            'name': 'action',
            'message': 'Please select a backup action:',
            'choices': ['Back', 'Setup', 'Advertise', 'Posts'],
            'filter': lambda val: str(val).lower()
        }
        answers = prompt(menu_prompt)
        return answers['action']

    @staticmethod
    def menu():
        action = Profile.ask_action()
        if (action == 'Back'): 
            from OnlySnarf.bin.menu import Menu
            return Menu.main_menu()
        elif (action == 'backup'): Profile.backup_menu()
        elif (action == 'check'): Profile.check()
        elif (action == 'posts'): Profile.posts_menu()
        elif (action == 'setup'): Profile.setup()
        elif (action == 'sync'): Profile.sync_from_profile()
        # elif (action == 'sync to'): Profile.sync_to_profile()
        
    def ask_action():
        menu_prompt = {
            'type': 'list',
            'name': 'action',
            'message': 'Please select a profile action:',
            'choices': ['Back', 'Backup','Sync', 
                # 'Sync To',
                'Check',
                'Posts',
                'Setup'
            ],
            'filter': lambda val: str(val).lower()
        }
        answers = prompt(menu_prompt)
        return answers['action']

    @staticmethod
    def create():
        if not Settings.is_prompt():
            return Profile()
        if not Settings.prompt("create profile"):
            return Profile()
        profile = Profile()
        print("Follow the prompts to create a Profile of settings.")
        print("Enter 'n' or nothing to not change the value.")
        for key, value in profile.items():
            question = {
                'type': 'input',
                'message': '{}:'.format(str(key).title()),
                'name': 'answer'
            }
            value_ = prompt(question)["answer"]
            if str(value_) == "n" or str(value_) == "" or str(value_) == " ": continue
            setattr(profile, str(key), value_)
        return profile

    @staticmethod
    def sync_from_profile():
        # opens every settings tab in the browser from pages or all
        # gets necessary variables from browser
        # variables = get_settings_variables()
        print("Syncing from Profile")
        profile = Profile()
        for tab in Profile.TABS:
            profile.sync_from_tab(tab)
        print("Synced from Profile")
        Profile.write_local(profile)
        return profile

    @staticmethod
    def sync_to_profile(profile=None):
        # syncs profile settings to onlyfans
        print("Syncing to Profile")
        if not profile:
            profile = Profile.read_local() or Profile.create()
        for tab in Profile.TABS:
            profile.sync_to_tab(tab)
        print("Synced to Profile")

    def sync_from_tab(self, tab):
        # syncs profile settings from the specificed tab
        Driver.get_driver().sync_from_settings_page(profile=self, page=tab)

    def sync_to_tab(self, tab):
        # syncs profile settings to the specificed tab
        Driver.get_driver().sync_to_settings_page(profile=self, page=tab)

    @staticmethod
    def get_country_list():
        return ["USA","Canada"]

    @staticmethod
    def get_variables_for_page(page):
        variables = get_settings_variables()
        vars_ = []
        for var in variables:
            if str(var[1]) == str(page):
                vars_.append(var)
        return vars_

    @staticmethod
    def fill_data():
        prof = {
            "coverImage": None,
            "profilePhoto": None,
            "displayName": "",
            "subscriptionPrice": "4.99",
            "about": "",
            "location": "",
            "websiteURL": None,
            "wishlist":None,
            "twitter":None,
            "google":None,
            "welcomeMessage":None,
            "twoStepAuth":False,
            "username": "",
            "email": "",
            "password": "",
            "emailNotifs": False,
            "emailNotifsNewReferral": False,
            "emailNotifsNewStream": False,
            "emailNotifsNewSubscriber": False,
            "emailNotifsNewTip": False,
            "emailNotifsRenewal": False,
            "emailNotifsNewLikes": False,
            "emailNotifsNewPosts": False,
            "emailNotifsNewPrivMessages": False,
            "siteNotifs": False,
            "siteNotifsNewComment": False,
            "siteNotifsNewFavorite": False,
            "siteNotifsDiscounts": False,
            "siteNotifsNewSubscriber": False,
            "siteNotifsNewTip": False,
            "toastNotifs": False,
            "toastNotifsNewComment": False,
            "toastNotifsNewFavorite": False,
            "toastNotifsNewSubscriber": False,
            "toastNotifsNewTip": False,
            "fullyPrivate": False,
            "enableComments": False,
            "showFansCount": False,
            "showPostsTip": False,
            "publicFriendsList": False,
            "ipCountry": Profile.get_country_list(),
            "ipIP": "",
            "watermark": True,
            "watermarkPhoto": False,
            "watermarkVideo": False,
            "watermarkText": "",
            "liveServer": "",
            "liveServerKey": ""
        }
        if prof.get("username") and str(prof.get("username")) != "" and prof.get("watermarkText") == "":
            prof.set("watermarkText", "OnlyFans.com/{}".format(prof.get("username")))
        return prof

    @staticmethod
    def read_local():
        Settings.maybe_print("Getting Local Profile")
        profile = None
        try:
            profile_ = {}
            with open(str(Settings.get_profile_path())) as json_file:  
                profile_ = json.load(json_file)['profile']
            Settings.maybe_print("Loaded Local Profile")
            profile = Profile()
            for key, value in profile_:
                setattr(profile, str(key), value)
        except Exception as e:
            Settings.dev_print(e)
        return profile

    @staticmethod
    def write_local(profile=None):
        if profile is None:
            profile = Profile.sync_from_profile()
        print("Saving Profile Locally")
        Settings.maybe_print("local profile path: "+str(Settings.get_profile_path()))
        try:
            with open(str(Settings.get_profile_path()), 'w') as outfile:  
                json.dump({"profile":profile.__dict__}, outfile, indent=4, sort_keys=True)
        except FileNotFoundError:
            print("Error: Missing Profile File")
        except OSError:
            print("Error: Missing Profile Path")

    # returns list of settings and their classes
    # ["settingVariableName","pageProfile","inputType-text"]
    
def get_settings_variables():
    return [

        ### Profile ###

        ["coverImage","profile","file"],
        ["profilePhoto","profile","file"],
        ["username","profile","text"],
        ["displayName","profile","text"],
        ["subscriptionPrice","profile","text"],
        ["referralReward","dropdown"],
        ["about","profile","text"],
        ["location","profile","text"],
        ["websiteURL","profile","text"],

        #### Account / Advanced ###

        # id="input-email"
        ["email","advanced","text"],
        # id="old_password_input"
        ["password","advanced","text"],
        # id="new_password_input"
        ["newPassword","advanced","text"],
        # id="new_password2_input"
        ["confirmPassword","advanced","checkbox"],

        ### Chats / Messages ###
        # welcome message toggle
        # welcome message text
        # welcome message file
        # welcome message record voice, video
        # welcome message price
        # welcome message submit

        # hide outgoing message toggle
        # show full text of message in the notification email

        ### Notifications ###

        # id="push-notifications"
        ["emailNotifs","notifications","toggle"],
        # id="email-notifications"
        ["emailNotifsReferral","notifications","checkbox"],
        ["emailNotifsStream","notifications","toggle"],
        ["emailNotifsSubscriber","notifications","toggle"],
        ["emailNotifsTip","notifications","toggle"],
        ["emailNotifsRenewal","notifications","toggle"],
        # this is a dropdown
        ["emailNotifsLikes","notifications","dropdown"],
        ["emailNotifsPosts","notifications","toggle"],
        ["emailNotifsPrivMessages","notifications","toggle"],
        ["siteNotifs","notifications","toggle"],
        ["siteNotifsComment","notifications","toggle"],
        ["siteNotifsFavorite","notifications","toggle"],
        ["siteNotifsDiscounts","notifications","toggle"],
        ["siteNotifsSubscriber","notifications","toggle"],
        ["siteNotifsTip","notifications","toggle"],
        ["toastNotifsComment","notifications","toggle"],
        ["toastNotifsFavorite","notifications","toggle"],
        ["toastNotifsSubscriber","notifications","toggle"],
        ["toastNotifsTip","notifications","toggle"],

        ### Security ###

        ["fullyPrivate","security","checkbox"],
        ["enableComments","security","toggle"],
        ["showFansCount","security","toggle"],
        ["showPostsTip","security","toggle"],
        ["publicFriendsList","security","toggle"],
        ["ipCountry","security","list"],
        # id="input-blocked-ips"
        ["ipIP","security","list"],
        # id="hasWatermarkPhoto"
        ["watermarkPhoto","security","toggle"],
        # id="hasWatermarkVideo"
        ["watermarkVideo","security","toggle"],
        # placeholder="Watermark custom text"
        ["watermarkText","security","text"],

        ### Story ###
        # allow message replies - nobody
        # allow message replies - subscribers

        ### Other ###

        ["liveServer","other","text"],
        ["liveServerKey","other","text"],
        ["welcomeMessageToggle","other","toggle"],
        ["welcomeMessageText","other","text"],

    ]




#!/usr/bin/python
# setup & update script for config
import os
import sys
import json
import shutil
from OnlySnarf.src import google as Google
from OnlySnarf.src import driver as OnlySnarf
from OnlySnarf.src.settings import Settings
from OnlySnarf.src import colorize

def checkBothCreds():
    checkGoogle()
    checkOnlyFans()

# checks Google creds access
def checkGoogle():
    print("Checking Google Creds (uploads)")
    if not os.path.exists(Settings.get_google_path()):
        print("Missing Google Creds")
        print()
        return main()
    authed = Google.checkAuth()
    if authed:
        print("Google Auth Successful")
    else: 
        print("Google Auth Failure")

# checks OnlyFans login process
def checkOnlyFans():
    print("Checking OnlyFans Creds")
    if not os.path.exists(Settings.get_config_path()):
        print("Missing Config Path")
        return main()
    OnlySnarf.auth()
    OnlySnarf.exit()

def checkTwitter():
    print("Checking Twitter Creds")
    if not os.path.exists(Settings.get_config_path()):
        print("Missing Config Path")
        return main()
    OnlySnarf.auth()
    OnlySnarf.exit()

# function that creates the missing config
def createConfig():
    print("Preparing OnlySnarf Config")
    # ensure /opt/onlysnarf exists
    if not os.path.exists("/opt/onlysnarf"):
        print("Creating Missing Config Dir")
        try:
            os.makedirs("/opt/onlysnarf")
            print("Created OnlySnarf Root")
        except Exception as e:
            print(e)
            main()
    if not os.path.exists("/opt/onlysnarf/config.conf"):
        print("Copying Default Config")
        try:
            shutil.copyfile(os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), "../config.conf")), Settings.get_config_path())
            shutil.chown(Settings.get_config_path(), user=os.environ['USER'], group=os.environ['USER'])
            print("Created OnlySnarf Config")
        except Exception as e:
            print(e)
            main()
    else:
        print("OnlySnarf Config Exists")
        return True
    return False

# provides instructions for creating or refreshing google creds
def googleInstructions():
    print("[Google Instructions From README Go Here]")

# creates the config then prompts for missing credentials
def setupConfig():
    alreadyCreated = createConfig()
    if not alreadyCreated:
        updateConfig()

# receives input for Google login
def receiveGoogle():
    data = {}
    data['username'] = input('Google Username: ')
    data['password'] = input('Google Password: ')
    return data

def receiveOnlyFans():
    data = {}
    data['email'] = input('OnlyFans Email: ')
    data['username'] = input('OnlyFans Username: ')
    data['password'] = input('OnlyFans Password: ')
    return data

# receives input for Twitter login
def receiveTwitter():
    data = {}
    data['username'] = input('Twitter Username: ')
    data['password'] = input('Twitter Password: ')
    return data

# refreshes all creds
def refreshAll():
    removeConfig()
    setupConfig()
    removeGoogle()
    googleInstructions()

# removes config.conf
def removeConfig():
    print("Removing Config")
    # ensure /opt/onlysnarf exists
    if os.path.exists(Settings.get_config_path()):
        os.remove(Settings.get_config_path())
        print("Removed Config")
    else:
        print("Error: Failed to Find Config")

# removes google creds
def removeGoogle():
    print("Removing Google Creds")
    # ensure /opt/onlysnarf exists
    if os.path.exists(Settings.get_google_path()):
        os.remove(Settings.get_google_path())
        print("Removed Google Creds")
    else:
        print("Error: Failed to Find Google Creds")

# receives input for twitter login and saves to config.conf
def updateConfig():
    updateOnlyFans()
    updateGoogle()
    updateTwitter()

def updateOnlyFans():
    data = receiveOnlyFans()
    # update conf variables username and password
    # save the conf file
    import fileinput
    # Does a list of files, and
    # redirects STDOUT to the file in question
    for line in fileinput.input(Settings.get_config_path(), inplace = 1): 
        line.replace("email None", "username {}".format(data['email']))
        line.replace("username None", "username {}".format(data['username']))
        line.replace("password None", "password {}".format(data['password']))
        print(line)
    print("OnlyFans Config Updated")

def updateGoogle():
    data = receiveGoogle()
    # update conf variables username and password
    # save the conf file
    import fileinput
    # Does a list of files, and
    # redirects STDOUT to the file in question
    for line in fileinput.input(Settings.get_config_path(), inplace = 1): 
        line.replace("username_google None", "username {}".format(data['username']))
        line.replace("password_google None", "password {}".format(data['password']))
        print(line)
    print("Google Config Updated")

def updateTwitter():
    data = receiveTwitter()
    # update conf variables username and password
    # save the conf file
    import fileinput
    # Does a list of files, and
    # redirects STDOUT to the file in question
    for line in fileinput.input(Settings.get_config_path(), inplace = 1): 
        line.replace("username_twitter None", "username {}".format(data['username']))
        line.replace("password_twitter None", "password {}".format(data['password']))
        print(line)
    print("Twitter Config Updated")
    
# this script is supposed to have menu options for 
# ) creating the .conf file
# ) updating the .conf file
# ) instructions for creating the google creds
# ) a function for checking the google creds
# when ran in it should check for the .conf file and google_creds
def main():
    print("-- OnlySnarf Config --")
    print("------------------------------")
    if os.path.isfile(Settings.get_config_path()):
        print(colorize("[*] Config File", 'conf')+": "+colorize("True", 'green'))
        if str(Settings.get_email()) != "None":
            print(colorize("[-] OnlyFans Email", 'conf')+": "+colorize(Settings.get_email(), 'green'))
        else:
            print(colorize("[-] OnlyFans Email", 'conf')+": "+colorize("", 'red'))
        if str(Settings.get_password()) != "None":
            print(colorize("[-] OnlyFans Password", 'conf')+": "+colorize("******", 'green'))
        else:
            print(colorize("[-] OnlyFans Password", 'conf')+": "+colorize("", 'red'))
        if str(Settings.get_username()) != "None":
            print(colorize("[-] OnlyFans Username", 'conf')+": "+colorize(Settings.get_username(), 'green'))
        else:
            print(colorize("[-] OnlyFans Username", 'conf')+": "+colorize("", 'red'))
        if str(Settings.get_username_google()) != "None":
            print(colorize("[-] Google Username", 'conf')+": "+colorize(Settings.get_username_google(), 'green'))
        else:
            print(colorize("[-] Google Username", 'conf')+": "+colorize("", 'red'))
        if str(Settings.get_password_google()) != "None":
            print(colorize("[-] Google Password", 'conf')+": "+colorize("******", 'green'))
        else:
            print(colorize("[-] Google Password", 'conf')+": "+colorize("", 'red'))
        if str(Settings.get_username_twitter()) != "None":
            print(colorize("[-] Twitter Username", 'conf')+": "+colorize(Settings.get_username_google(), 'green'))
        else:
            print(colorize("[-] Twitter Username", 'conf')+": "+colorize("", 'red'))
        if str(Settings.get_password_twitter()) != "None":
            print(colorize("[-] Twitter Password", 'conf')+": "+colorize("******", 'green'))
        else:
            print(colorize("[-] Twitter Password", 'conf')+": "+colorize("", 'red'))
    else:
        print(colorize("[*] Config File", 'conf')+": "+colorize("False", 'red'))
    if os.path.isfile(Settings.get_google_path()):
        print(colorize("[*] Google Creds", 'conf')+": "+colorize("True", 'green'))
    else:
        print(colorize("[*] Google Creds", 'conf')+": "+colorize("False", 'red'))
    print("------------------------------")
    print(colorize("Menu:", 'menu'))
    print(colorize("[ 0 ]", 'menu') + " Config - Create")
    print(colorize("[ 1 ]", 'menu') + " Config - Update - Google")
    print(colorize("[ 2 ]", 'menu') + " Config - Update - OnlyFans")
    print(colorize("[ 3 ]", 'menu') + " Config - Update - Twitter")
    print(colorize("[ 4 ]", 'menu') + " Config - Remove")
    print(colorize("[ 5 ]", 'menu') + " Google Creds - Check")
    print(colorize("[ 6 ]", 'menu') + " Google Creds - Instructions")
    print(colorize("[ 7 ]", 'menu') + " Google Creds - Remove")
    # print(colorize("[ 8 ]", 'menu') + " Refresh All")
    while True:
        choice = input(">> ")
        try:
            if int(choice) < 0 or int(choice) >= 9: raise ValueError
            # elif int(choice) == 2:
            #     checkBothCreds()
            if int(choice) == 0:
                setupConfig()
            elif int(choice) == 1:
                updateGoogle()
            elif int(choice) == 2:
                updateOnlyFans()
            elif int(choice) == 3:
                updateTwitter()
            elif int(choice) == 4:
                removeConfig()
            elif int(choice) == 5:
                checkGoogle()
            elif int(choice) == 6:
                googleInstructions()
            elif int(choice) == 7:
                removeGoogle()
            # elif int(choice) == 8:
            #     refreshAll()
        except (ValueError, IndexError, KeyboardInterrupt):
            print("Error: Incorrect Index")
    print()
    main()

###########################

if __name__ == "__main__":
    try:
        Settings.initialize()
        main()
    except Exception as e:
        Settings.maybePrint(e)
        print(e)